/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package flashcards4;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 *
 * @author pinter
 */
public class FlashcardsGUI {

    ArrayList<Card> cards;
    private int cardNumber;
    private boolean showQuestion;
    private int maxScore;
    private int score;
    private ArrayList<Integer> cardFrequencies;
    private final int STARTING_FREQUENCY = 5;
    private boolean ended;

    private JFrame frame;
    private JPanel northPanel;
    private JPanel southPanel;
    private JTextArea display;
    private JLabel scoreLabel;

    public FlashcardsGUI() {
        frame = new JFrame("Flashcards");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        northPanel = new JPanel();
        display = new JTextArea("Please open a cards file.", 10, 40);
        display.setEditable(false);
        northPanel.add(display);
        frame.getContentPane().add(northPanel, BorderLayout.NORTH);

        southPanel = new JPanel();
        scoreLabel = new JLabel("0/0");
        southPanel.add(scoreLabel);
        String[] buttonLabels = {"Reset", "Toogle Q/A", "Wrong answer", "Good answer"};
        ActionListener[] listeners = {new ResetActionListener(),
            new ToggleActionListener(), new AnswerActionListener(false),
            new AnswerActionListener(true)};
        for (int i = 0; i < buttonLabels.length; ++i) {
            JButton button = new JButton(buttonLabels[i]);
            button.addActionListener(listeners[i]);
            southPanel.add(button);
        }
        frame.getContentPane().add(southPanel, BorderLayout.SOUTH);

        JMenuBar menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);
        JMenu flashcardsMenu = new JMenu("File");
        menuBar.add(flashcardsMenu);
        JMenuItem openMenuItem = new JMenuItem("Open...");
        openMenuItem.addActionListener(new OpenActionListener());
        flashcardsMenu.add(openMenuItem);
        JMenuItem exitMenuItem = new JMenuItem("Exit");
        flashcardsMenu.add(exitMenuItem);
        exitMenuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                System.exit(0);
            }
        });

        frame.pack();
        frame.setVisible(true);
    }

    public void reset() {
        cardNumber = 0;
        score = 0;
        maxScore = 0;
        showQuestion = true;
        ended = false;
        cardFrequencies = new ArrayList<>();
        for (int i = 0; i < cards.size(); ++i) {
            cardFrequencies.add(STARTING_FREQUENCY);
        }
        displayScore();
        displayCard();
    }

    private void displayScore() {
        scoreLabel.setText(score + "/" + maxScore);
    }

    private void displayCard() {
        if (cards != null && !ended) {
            display.setText(showQuestion
                    ? cards.get(cardNumber).getQuestion()
                    : cards.get(cardNumber).getAnswer());
        } else if (cards != null) {
            display.setText("Congratulations!");
        }

    }

    class OpenActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            final JFileChooser fc = new JFileChooser();
            int returnVal = fc.showOpenDialog(frame);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                String line;
                try (BufferedReader br = new BufferedReader(new FileReader(file))) {
                    cards = new ArrayList<>();
                    cardFrequencies = new ArrayList<>();
                    boolean error = false;
                    while ((line = br.readLine()) != null) {
                        String qa[] = line.split("~");
                        if (qa.length == 2) {
                            cards.add(new Card(qa[0], qa[1]));
                        } else {
                            error = true;
                        }
                    }
                    if (error) {
                        JOptionPane.showMessageDialog(frame, "There were incorrectly formatted lines in the file.",
                                "Warning", JOptionPane.WARNING_MESSAGE);

                    }
                    if (cards.isEmpty()) {
                        cards = null;
                    }
                    reset();
                } catch (FileNotFoundException ex) {
                    JOptionPane.showMessageDialog(frame, "File not found!",
                            "Error", JOptionPane.ERROR_MESSAGE);
                } catch (IOException ex) {
                    JOptionPane.showMessageDialog(frame, "The format of the file is wrong!",
                            "Error", JOptionPane.ERROR_MESSAGE);
                }
            }

        }

    }

    class ResetActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            reset();
        }

    }

    class ToggleActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            showQuestion = !showQuestion;
            displayCard();
        }

    }

    class AnswerActionListener implements ActionListener {

        private Random random;
        private boolean incScore;

        public AnswerActionListener(boolean incScore) {
            this.incScore = incScore;
            random = new Random();
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if (cards != null && !ended) {
                if (incScore) {
                    score++;
                    cardFrequencies.set(cardNumber, cardFrequencies.get(cardNumber) - 1);
                } else {
                    cardFrequencies.set(cardNumber, cardFrequencies.get(cardNumber) + 1);
                }
                maxScore++;
                displayScore();
                int maxFrequency = 0;
                for (Integer k : cardFrequencies) {
                    maxFrequency += k;
                }
                if (maxFrequency == 0) {
                    ended = true;
                    displayCard();
                    return;
                }
                int nextCardNumber = 0;
                int randomIndex = random.nextInt(maxFrequency);
                int sumFrequencies = 0;
                for (int i = 0; i < cardFrequencies.size(); ++i) {
                    sumFrequencies += cardFrequencies.get(i);
                    if (sumFrequencies >= randomIndex) {
                        nextCardNumber = i;
                        break;
                    }
                }
                cardNumber = nextCardNumber;
                showQuestion = true;
                displayCard();
            }
        }

    }

}
